from collections import defaultdict
from skidl import Pin, Part, Alias, SchLib, SKIDL, TEMPLATE

from skidl.pin import pin_types

SKIDL_lib_version = '0.0.1'

Amplifier_Difference = SchLib(tool=SKIDL).add_parts(*[
        Part(**{ 'name':'AD628', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'AD628'}), 'ref_prefix':'U', 'fplist':[''], 'footprint':'', 'keywords':'difference amplifier', 'description':'', 'datasheet':'https://www.analog.com/media/en/technical-documentation/data-sheets/AD628.pdf', 'search_text':'/usr/share/kicad/symbols/Amplifier_Difference.kicad_sym\nAD628\n\ndifference amplifier', 'pins':[
            Pin(num='1',name='+',func=pin_types.INPUT,unit=1),
            Pin(num='2',name='V-',func=pin_types.PWRIN,unit=1),
            Pin(num='3',name='VREF',func=pin_types.PASSIVE,unit=1),
            Pin(num='4',name='CFILT',func=pin_types.PASSIVE,unit=1),
            Pin(num='5',name='~',func=pin_types.OUTPUT,unit=1),
            Pin(num='6',name='RG',func=pin_types.PASSIVE,unit=1),
            Pin(num='7',name='V+',func=pin_types.PWRIN,unit=1),
            Pin(num='8',name='-',func=pin_types.INPUT,unit=1)], 'unit_defs':[] }),
        Part(**{ 'name':'AD8207', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'AD8207'}), 'ref_prefix':'U', 'fplist':['Package_SO:SOIC-8_3.9x4.9mm_P1.27mm'], 'footprint':'Package_SO:SOIC-8_3.9x4.9mm_P1.27mm', 'keywords':'highside HS current sense difference amplifier linear buffered', 'description':'', 'datasheet':'https://www.analog.com/media/en/technical-documentation/data-sheets/AD8207.pdf', 'search_text':'/usr/share/kicad/symbols/Amplifier_Difference.kicad_sym\nAD8207\n\nhighside HS current sense difference amplifier linear buffered', 'pins':[
            Pin(num='1',name='-IN',func=pin_types.INPUT,unit=1),
            Pin(num='2',name='GND',func=pin_types.PWRIN,unit=1),
            Pin(num='3',name='Vref2',func=pin_types.INPUT,unit=1),
            Pin(num='4',name='RANGE',func=pin_types.INPUT,unit=1),
            Pin(num='5',name='OUT',func=pin_types.OUTPUT,unit=1),
            Pin(num='6',name='V+',func=pin_types.PWRIN,unit=1),
            Pin(num='7',name='Vref1',func=pin_types.INPUT,unit=1),
            Pin(num='8',name='+IN',func=pin_types.INPUT,unit=1)], 'unit_defs':[] }),
        Part(**{ 'name':'AD8276', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'AD8276'}), 'ref_prefix':'U', 'fplist':[''], 'footprint':'', 'keywords':'difference amplifier', 'description':'', 'datasheet':'https://www.analog.com/media/en/technical-documentation/data-sheets/AD8276_8277.pdf', 'search_text':'/usr/share/kicad/symbols/Amplifier_Difference.kicad_sym\nAD8276\n\ndifference amplifier', 'pins':[
            Pin(num='1',name='REF',func=pin_types.PASSIVE,unit=1),
            Pin(num='2',name='-',func=pin_types.PASSIVE,unit=1),
            Pin(num='3',name='+',func=pin_types.PASSIVE,unit=1),
            Pin(num='4',name='V-',func=pin_types.PWRIN,unit=1),
            Pin(num='5',name='SENSE',func=pin_types.PASSIVE,unit=1),
            Pin(num='6',name='~',func=pin_types.OUTPUT,unit=1),
            Pin(num='7',name='V+',func=pin_types.PWRIN,unit=1),
            Pin(num='8',name='~',func=pin_types.NOCONNECT,unit=1)], 'unit_defs':[] }),
        Part(**{ 'name':'AD8475ACPZ', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'AD8475ACPZ'}), 'ref_prefix':'U', 'fplist':['Package_CSP:LFCSP-16-1EP_3x3mm_P0.5mm_EP1.6x1.6mm'], 'footprint':'Package_CSP:LFCSP-16-1EP_3x3mm_P0.5mm_EP1.6x1.6mm', 'keywords':'selectable gain ADC driver', 'description':'', 'datasheet':'https://www.analog.com/media/en/technical-documentation/data-sheets/AD8475.pdf', 'search_text':'/usr/share/kicad/symbols/Amplifier_Difference.kicad_sym\nAD8475ACPZ\n\nselectable gain ADC driver', 'pins':[
            Pin(num='1',name='+IN0.4x',func=pin_types.INPUT,unit=1),
            Pin(num='10',name='+',func=pin_types.OUTPUT,unit=1),
            Pin(num='11',name='-',func=pin_types.OUTPUT,unit=1),
            Pin(num='12',name='NC',func=pin_types.NOCONNECT,unit=1),
            Pin(num='13',name='-Vs',func=pin_types.PWRIN,unit=1),
            Pin(num='14',name='-Vs',func=pin_types.PASSIVE,unit=1),
            Pin(num='15',name='-Vs',func=pin_types.PASSIVE,unit=1),
            Pin(num='16',name='+IN0.4x',func=pin_types.INPUT,unit=1),
            Pin(num='17',name='EP',func=pin_types.PASSIVE,unit=1),
            Pin(num='2',name='+IN0.8x',func=pin_types.INPUT,unit=1),
            Pin(num='3',name='-IN0.8x',func=pin_types.INPUT,unit=1),
            Pin(num='4',name='-IN0.4x',func=pin_types.INPUT,unit=1),
            Pin(num='5',name='-IN0.4x',func=pin_types.INPUT,unit=1),
            Pin(num='6',name='+Vs',func=pin_types.PWRIN,unit=1),
            Pin(num='7',name='+Vs',func=pin_types.PASSIVE,unit=1),
            Pin(num='8',name='+Vs',func=pin_types.PASSIVE,unit=1),
            Pin(num='9',name='VOCM',func=pin_types.PWRIN,unit=1)], 'unit_defs':[] }),
        Part(**{ 'name':'AD8475xRMZ', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'AD8475xRMZ'}), 'ref_prefix':'U', 'fplist':['Package_SO:MSOP-10_3x3mm_P0.5mm'], 'footprint':'Package_SO:MSOP-10_3x3mm_P0.5mm', 'keywords':'selectable gain ADC driver', 'description':'', 'datasheet':'https://www.analog.com/media/en/technical-documentation/data-sheets/AD8475.pdf', 'search_text':'/usr/share/kicad/symbols/Amplifier_Difference.kicad_sym\nAD8475xRMZ\n\nselectable gain ADC driver', 'pins':[
            Pin(num='1',name='-IN0.8x',func=pin_types.INPUT,unit=1),
            Pin(num='10',name='+IN0.8x',func=pin_types.INPUT,unit=1),
            Pin(num='2',name='-IN0.4x',func=pin_types.INPUT,unit=1),
            Pin(num='3',name='+Vs',func=pin_types.PWRIN,unit=1),
            Pin(num='4',name='VOCM',func=pin_types.PWRIN,unit=1),
            Pin(num='5',name='+',func=pin_types.OUTPUT,unit=1),
            Pin(num='6',name='-',func=pin_types.OUTPUT,unit=1),
            Pin(num='7',name='NC',func=pin_types.NOCONNECT,unit=1),
            Pin(num='8',name='-Vs',func=pin_types.PWRIN,unit=1),
            Pin(num='9',name='+IN0.4x',func=pin_types.INPUT,unit=1)], 'unit_defs':[] }),
        Part(**{ 'name':'ADA4938-1', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'ADA4938-1'}), 'ref_prefix':'U', 'fplist':['Package_CSP:LFCSP-16-1EP_3x3mm_P0.5mm_EP1.3x1.3mm_ThermalVias'], 'footprint':'Package_CSP:LFCSP-16-1EP_3x3mm_P0.5mm_EP1.3x1.3mm_ThermalVias', 'keywords':'Fully-Differential Amplifier ADC Driver', 'description':'', 'datasheet':'https://www.analog.com/media/en/technical-documentation/data-sheets/ADA4938-1_4938-2.pdf', 'search_text':'/usr/share/kicad/symbols/Amplifier_Difference.kicad_sym\nADA4938-1\n\nFully-Differential Amplifier ADC Driver', 'pins':[
            Pin(num='1',name='FB-',func=pin_types.OUTPUT,unit=1),
            Pin(num='10',func=pin_types.OUTPUT,unit=1),
            Pin(num='11',func=pin_types.OUTPUT,unit=1),
            Pin(num='2',name='+',func=pin_types.INPUT,unit=1),
            Pin(num='3',name='-',func=pin_types.INPUT,unit=1),
            Pin(num='4',name='FB+',func=pin_types.OUTPUT,unit=1),
            Pin(num='9',name='V_{OCM}',func=pin_types.INPUT,unit=1),
            Pin(num='12',name='~{PD}',func=pin_types.INPUT,unit=2),
            Pin(num='13',name='V-',func=pin_types.PWRIN,unit=2),
            Pin(num='14',name='V-',func=pin_types.PASSIVE,unit=2),
            Pin(num='15',name='V-',func=pin_types.PASSIVE,unit=2),
            Pin(num='16',name='V-',func=pin_types.PASSIVE,unit=2),
            Pin(num='17',name='PAD',func=pin_types.PASSIVE,unit=2),
            Pin(num='5',name='V+',func=pin_types.PWRIN,unit=2),
            Pin(num='6',name='V+',func=pin_types.PASSIVE,unit=2),
            Pin(num='7',name='V+',func=pin_types.PASSIVE,unit=2),
            Pin(num='8',name='V+',func=pin_types.PASSIVE,unit=2)], 'unit_defs':[{'label': 'uA', 'num': 1, 'pin_nums': ['2', '9', '10', '3', '11', '1', '4']},{'label': 'uB', 'num': 2, 'pin_nums': ['17', '7', '16', '13', '6', '15', '5', '8', '12', '14']}] }),
        Part(**{ 'name':'ADA4940-1xCP', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'ADA4940-1xCP'}), 'ref_prefix':'U', 'fplist':['Package_CSP:LFCSP-16-1EP_3x3mm_P0.5mm_EP1.3x1.3mm_ThermalVias'], 'footprint':'Package_CSP:LFCSP-16-1EP_3x3mm_P0.5mm_EP1.3x1.3mm_ThermalVias', 'keywords':'differential amplifier', 'description':'', 'datasheet':'https://www.analog.com/media/en/technical-documentation/data-sheets/ADA4940-1_4940-2.pdf', 'search_text':'/usr/share/kicad/symbols/Amplifier_Difference.kicad_sym\nADA4940-1xCP\n\ndifferential amplifier', 'pins':[
            Pin(num='1',name='-FB',func=pin_types.INPUT,unit=1),
            Pin(num='10',name='+OUT',func=pin_types.OUTPUT,unit=1),
            Pin(num='11',name='-OUT',func=pin_types.OUTPUT,unit=1),
            Pin(num='12',name='~{DISABLE}',func=pin_types.INPUT,unit=1),
            Pin(num='2',name='+IN',func=pin_types.INPUT,unit=1),
            Pin(num='3',name='-IN',func=pin_types.INPUT,unit=1),
            Pin(num='4',name='+FB',func=pin_types.INPUT,unit=1),
            Pin(num='9',name='VOCM',func=pin_types.INPUT,unit=1),
            Pin(num='13',name='-Vs',func=pin_types.PWRIN,unit=2),
            Pin(num='14',name='-Vs',func=pin_types.PASSIVE,unit=2),
            Pin(num='15',name='-Vs',func=pin_types.PASSIVE,unit=2),
            Pin(num='16',name='-Vs',func=pin_types.PASSIVE,unit=2),
            Pin(num='17',name='EP',func=pin_types.PASSIVE,unit=2),
            Pin(num='5',name='+Vs',func=pin_types.PWRIN,unit=2),
            Pin(num='6',name='+Vs',func=pin_types.PASSIVE,unit=2),
            Pin(num='7',name='+Vs',func=pin_types.PASSIVE,unit=2),
            Pin(num='8',name='+Vs',func=pin_types.PASSIVE,unit=2)], 'unit_defs':[{'label': 'uA', 'num': 1, 'pin_nums': ['1', '11', '12', '4', '2', '10', '9', '3']},{'label': 'uB', 'num': 2, 'pin_nums': ['14', '13', '6', '17', '15', '7', '5', '8', '16']}] }),
        Part(**{ 'name':'ADA4940-2', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'ADA4940-2'}), 'ref_prefix':'U', 'fplist':['Package_CSP:LFCSP-24-1EP_4x4mm_P0.5mm_EP2.5x2.5mm'], 'footprint':'Package_CSP:LFCSP-24-1EP_4x4mm_P0.5mm_EP2.5x2.5mm', 'keywords':'differential amplifier', 'description':'', 'datasheet':'https://www.analog.com/media/en/technical-documentation/data-sheets/ADA4940-1_4940-2.pdf', 'search_text':'/usr/share/kicad/symbols/Amplifier_Difference.kicad_sym\nADA4940-2\n\ndifferential amplifier', 'pins':[
            Pin(num='1',name='-IN1',func=pin_types.INPUT,unit=1),
            Pin(num='17',name='VOCM1',func=pin_types.INPUT,unit=1),
            Pin(num='18',name='+1',func=pin_types.OUTPUT,unit=1),
            Pin(num='19',name='-1',func=pin_types.OUTPUT,unit=1),
            Pin(num='2',name='+FB1',func=pin_types.INPUT,unit=1),
            Pin(num='20',name='~{DISABLE1}',func=pin_types.INPUT,unit=1),
            Pin(num='23',name='-FB1',func=pin_types.INPUT,unit=1),
            Pin(num='24',name='+IN1',func=pin_types.INPUT,unit=1),
            Pin(num='11',name='VOCM2',func=pin_types.INPUT,unit=2),
            Pin(num='12',name='+2',func=pin_types.OUTPUT,unit=2),
            Pin(num='13',name='-2',func=pin_types.OUTPUT,unit=2),
            Pin(num='14',name='~{DISABLE2}',func=pin_types.INPUT,unit=2),
            Pin(num='5',name='-FB2',func=pin_types.INPUT,unit=2),
            Pin(num='6',name='+IN2',func=pin_types.INPUT,unit=2),
            Pin(num='7',name='-IN2',func=pin_types.INPUT,unit=2),
            Pin(num='8',name='+FB2',func=pin_types.INPUT,unit=2),
            Pin(num='21',name='-FB2',func=pin_types.INPUT,unit=3),
            Pin(num='22',name='-FB2',func=pin_types.INPUT,unit=3),
            Pin(num='25',name='EP',func=pin_types.PASSIVE,unit=3),
            Pin(num='3',name='+FB2',func=pin_types.INPUT,unit=3),
            Pin(num='4',name='+FB2',func=pin_types.INPUT,unit=3),
            Pin(num='10',name='+FB2',func=pin_types.INPUT,unit=4),
            Pin(num='15',name='-FB2',func=pin_types.INPUT,unit=4),
            Pin(num='16',name='-FB2',func=pin_types.INPUT,unit=4),
            Pin(num='9',name='+FB2',func=pin_types.INPUT,unit=4)], 'unit_defs':[{'label': 'uA', 'num': 1, 'pin_nums': ['23', '18', '1', '2', '17', '24', '20', '19']},{'label': 'uB', 'num': 2, 'pin_nums': ['11', '12', '6', '14', '8', '7', '5', '13']},{'label': 'uC', 'num': 3, 'pin_nums': ['21', '25', '22', '4', '3']},{'label': 'uD', 'num': 4, 'pin_nums': ['10', '16', '15', '9']}] }),
        Part(**{ 'name':'AMC1100DWV', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'AMC1100DWV'}), 'ref_prefix':'U', 'fplist':['Package_SO:SOIC-8_7.5x5.85mm_P1.27mm'], 'footprint':'Package_SO:SOIC-8_7.5x5.85mm_P1.27mm', 'keywords':'isolated difference amplifier', 'description':'', 'datasheet':'http://www.ti.com/lit/ds/symlink/amc1100.pdf', 'search_text':'/usr/share/kicad/symbols/Amplifier_Difference.kicad_sym\nAMC1100DWV\n\nisolated difference amplifier', 'pins':[
            Pin(num='1',name='V+',func=pin_types.PWRIN,unit=1),
            Pin(num='2',name='+',func=pin_types.INPUT,unit=1),
            Pin(num='3',name='-',func=pin_types.INPUT,unit=1),
            Pin(num='4',name='V-',func=pin_types.PWRIN,unit=1),
            Pin(num='5',name='V-',func=pin_types.PWRIN,unit=1),
            Pin(num='6',name='-',func=pin_types.OUTPUT,unit=1),
            Pin(num='7',name='+',func=pin_types.OUTPUT,unit=1),
            Pin(num='8',name='V+',func=pin_types.PWRIN,unit=1)], 'unit_defs':[] }),
        Part(**{ 'name':'LM733CH', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'LM733CH'}), 'ref_prefix':'U', 'fplist':['Package_TO_SOT_THT:TO-5-10'], 'footprint':'Package_TO_SOT_THT:TO-5-10', 'keywords':'single differential video opamp', 'description':'', 'datasheet':'http://www.soemtron.org/downloads/disposals/lm733cn.pdf', 'search_text':'/usr/share/kicad/symbols/Amplifier_Difference.kicad_sym\nLM733CH\n\nsingle differential video opamp', 'pins':[
            Pin(num='1',name='~',func=pin_types.INPUT,unit=1),
            Pin(num='10',name='2A',func=pin_types.PASSIVE,unit=1),
            Pin(num='2',name='~',func=pin_types.INPUT,unit=1),
            Pin(num='3',name='2B',func=pin_types.PASSIVE,unit=1),
            Pin(num='4',name='1B',func=pin_types.PASSIVE,unit=1),
            Pin(num='5',name='V-',func=pin_types.PWRIN,unit=1),
            Pin(num='6',name='~',func=pin_types.OUTPUT,unit=1),
            Pin(num='7',name='~',func=pin_types.OUTPUT,unit=1),
            Pin(num='8',name='V+',func=pin_types.PWRIN,unit=1),
            Pin(num='9',name='1A',func=pin_types.PASSIVE,unit=1)], 'unit_defs':[] }),
        Part(**{ 'name':'LM733CN', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'LM733CN'}), 'ref_prefix':'U', 'fplist':['Package_DIP:DIP-14_W7.62mm'], 'footprint':'Package_DIP:DIP-14_W7.62mm', 'keywords':'single differential video opamp', 'description':'', 'datasheet':'http://www.soemtron.org/downloads/disposals/lm733cn.pdf', 'search_text':'/usr/share/kicad/symbols/Amplifier_Difference.kicad_sym\nLM733CN\n\nsingle differential video opamp', 'pins':[
            Pin(num='1',name='~',func=pin_types.INPUT,unit=1),
            Pin(num='10',name='V+',func=pin_types.PWRIN,unit=1),
            Pin(num='11',name='1A',func=pin_types.PASSIVE,unit=1),
            Pin(num='12',name='2A',func=pin_types.PASSIVE,unit=1),
            Pin(num='13',name='NC',func=pin_types.NOCONNECT,unit=1),
            Pin(num='14',name='~',func=pin_types.INPUT,unit=1),
            Pin(num='2',name='NC',func=pin_types.NOCONNECT,unit=1),
            Pin(num='3',name='2B',func=pin_types.PASSIVE,unit=1),
            Pin(num='4',name='1B',func=pin_types.PASSIVE,unit=1),
            Pin(num='5',name='V-',func=pin_types.PWRIN,unit=1),
            Pin(num='6',name='NC',func=pin_types.NOCONNECT,unit=1),
            Pin(num='7',name='~',func=pin_types.OUTPUT,unit=1),
            Pin(num='8',name='~',func=pin_types.OUTPUT,unit=1),
            Pin(num='9',name='NC',func=pin_types.NOCONNECT,unit=1)], 'unit_defs':[] }),
        Part(**{ 'name':'LTC1992-x-xMS8', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'LTC1992-x-xMS8'}), 'ref_prefix':'U', 'fplist':['Package_SO:MSOP-8_3x3mm_P0.65mm'], 'footprint':'Package_SO:MSOP-8_3x3mm_P0.65mm', 'keywords':'fully differential amplifier', 'description':'', 'datasheet':'https://www.analog.com/media/en/technical-documentation/data-sheets/1992fb.pdf', 'search_text':'/usr/share/kicad/symbols/Amplifier_Difference.kicad_sym\nLTC1992-x-xMS8\n\nfully differential amplifier', 'pins':[
            Pin(num='1',name='-',func=pin_types.INPUT,unit=1),
            Pin(num='2',name='V_{OCM}',func=pin_types.INPUT,unit=1),
            Pin(num='3',name='+V_{S}',func=pin_types.PWRIN,unit=1),
            Pin(num='4',func=pin_types.OUTPUT,unit=1),
            Pin(num='5',func=pin_types.OUTPUT,unit=1),
            Pin(num='6',name='-V_{S}',func=pin_types.PWRIN,unit=1),
            Pin(num='7',name='V_{Mid}',func=pin_types.INPUT,unit=1),
            Pin(num='8',name='+',func=pin_types.INPUT,unit=1)], 'unit_defs':[] }),
        Part(**{ 'name':'THS4521IDGK', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'THS4521IDGK'}), 'ref_prefix':'U', 'fplist':['Package_SO:VSSOP-8_3x3mm_P0.65mm'], 'footprint':'Package_SO:VSSOP-8_3x3mm_P0.65mm', 'keywords':'differential amplifier', 'description':'', 'datasheet':'https://www.ti.com/lit/ds/symlink/ths4521.pdf', 'search_text':'/usr/share/kicad/symbols/Amplifier_Difference.kicad_sym\nTHS4521IDGK\n\ndifferential amplifier', 'pins':[
            Pin(num='1',name='-',func=pin_types.INPUT,unit=1),
            Pin(num='2',name='V_{OCM}',func=pin_types.INPUT,unit=1),
            Pin(num='3',name='V_{S+}',func=pin_types.PWRIN,unit=1),
            Pin(num='4',func=pin_types.OUTPUT,unit=1),
            Pin(num='5',func=pin_types.OUTPUT,unit=1),
            Pin(num='6',name='V_{S-}',func=pin_types.PWRIN,unit=1),
            Pin(num='7',name='~{PD}',func=pin_types.INPUT,unit=1),
            Pin(num='8',name='+',func=pin_types.INPUT,unit=1)], 'unit_defs':[] }),
        Part(**{ 'name':'THS4551xRGT', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'THS4551xRGT'}), 'ref_prefix':'U', 'fplist':['Package_DFN_QFN:VQFN-16-1EP_3x3mm_P0.5mm_EP1.68x1.68mm'], 'footprint':'Package_DFN_QFN:VQFN-16-1EP_3x3mm_P0.5mm_EP1.68x1.68mm', 'keywords':'differential amplifier', 'description':'', 'datasheet':'https://www.ti.com/lit/ds/symlink/ths4551.pdf', 'search_text':'/usr/share/kicad/symbols/Amplifier_Difference.kicad_sym\nTHS4551xRGT\n\ndifferential amplifier', 'pins':[
            Pin(num='1',name='FB-',func=pin_types.OUTPUT,unit=1),
            Pin(num='10',name='OUT+',func=pin_types.OUTPUT,unit=1),
            Pin(num='11',name='OUT-',func=pin_types.OUTPUT,unit=1),
            Pin(num='12',name='~{PD}',func=pin_types.INPUT,unit=1),
            Pin(num='2',name='IN+',func=pin_types.INPUT,unit=1),
            Pin(num='3',name='IN-',func=pin_types.INPUT,unit=1),
            Pin(num='4',name='FB+',func=pin_types.OUTPUT,unit=1),
            Pin(num='9',name='VOCM',func=pin_types.INPUT,unit=1),
            Pin(num='13',name='VS-',func=pin_types.PWRIN,unit=2),
            Pin(num='14',name='VS-',func=pin_types.PASSIVE,unit=2),
            Pin(num='15',name='VS-',func=pin_types.PASSIVE,unit=2),
            Pin(num='16',name='VS-',func=pin_types.PASSIVE,unit=2),
            Pin(num='17',name='EP',func=pin_types.PASSIVE,unit=2),
            Pin(num='5',name='VS+',func=pin_types.PWRIN,unit=2),
            Pin(num='6',name='VS+',func=pin_types.PASSIVE,unit=2),
            Pin(num='7',name='VS+',func=pin_types.PASSIVE,unit=2),
            Pin(num='8',name='VS+',func=pin_types.PASSIVE,unit=2)], 'unit_defs':[{'label': 'uA', 'num': 1, 'pin_nums': ['9', '3', '11', '12', '4', '10', '2', '1']},{'label': 'uB', 'num': 2, 'pin_nums': ['14', '16', '17', '7', '5', '13', '6', '15', '8']}] }),
        Part(**{ 'name':'AMC1200BDWV', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'AMC1200BDWV'}), 'ref_prefix':'U', 'fplist':['Package_SO:SOIC-8_7.5x5.85mm_P1.27mm', 'Package_SO:SOIC-8_7.5x5.85mm_P1.27mm'], 'footprint':'Package_SO:SOIC-8_7.5x5.85mm_P1.27mm', 'keywords':'isolated difference amplifier', 'description':'', 'datasheet':'http://www.ti.com/lit/ds/symlink/amc1200.pdf', 'search_text':'/usr/share/kicad/symbols/Amplifier_Difference.kicad_sym\nAMC1200BDWV\n\nisolated difference amplifier', 'pins':[
            Pin(num='1',name='V+',func=pin_types.PWRIN,unit=1),
            Pin(num='2',name='+',func=pin_types.INPUT,unit=1),
            Pin(num='3',name='-',func=pin_types.INPUT,unit=1),
            Pin(num='4',name='V-',func=pin_types.PWRIN,unit=1),
            Pin(num='5',name='V-',func=pin_types.PWRIN,unit=1),
            Pin(num='6',name='-',func=pin_types.OUTPUT,unit=1),
            Pin(num='7',name='+',func=pin_types.OUTPUT,unit=1),
            Pin(num='8',name='V+',func=pin_types.PWRIN,unit=1)], 'unit_defs':[] }),
        Part(**{ 'name':'AMC1300BDWV', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'AMC1300BDWV'}), 'ref_prefix':'U', 'fplist':['Package_SO:SOIC-8_7.5x5.85mm_P1.27mm', 'Package_SO:SOIC-8_7.5x5.85mm_P1.27mm', 'Package_SO:SOIC-8_7.5x5.85mm_P1.27mm'], 'footprint':'Package_SO:SOIC-8_7.5x5.85mm_P1.27mm', 'keywords':'isolated difference amplifier', 'description':'', 'datasheet':'http://www.ti.com/lit/ds/symlink/amc1300.pdf', 'search_text':'/usr/share/kicad/symbols/Amplifier_Difference.kicad_sym\nAMC1300BDWV\n\nisolated difference amplifier', 'pins':[
            Pin(num='1',name='V+',func=pin_types.PWRIN,unit=1),
            Pin(num='2',name='+',func=pin_types.INPUT,unit=1),
            Pin(num='3',name='-',func=pin_types.INPUT,unit=1),
            Pin(num='4',name='V-',func=pin_types.PWRIN,unit=1),
            Pin(num='5',name='V-',func=pin_types.PWRIN,unit=1),
            Pin(num='6',name='-',func=pin_types.OUTPUT,unit=1),
            Pin(num='7',name='+',func=pin_types.OUTPUT,unit=1),
            Pin(num='8',name='V+',func=pin_types.PWRIN,unit=1)], 'unit_defs':[] }),
        Part(**{ 'name':'AMC1300DWV', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'AMC1300DWV'}), 'ref_prefix':'U', 'fplist':['Package_SO:SOIC-8_7.5x5.85mm_P1.27mm', 'Package_SO:SOIC-8_7.5x5.85mm_P1.27mm', 'Package_SO:SOIC-8_7.5x5.85mm_P1.27mm', 'Package_SO:SOIC-8_7.5x5.85mm_P1.27mm'], 'footprint':'Package_SO:SOIC-8_7.5x5.85mm_P1.27mm', 'keywords':'isolated difference amplifier', 'description':'', 'datasheet':'http://www.ti.com/lit/ds/symlink/amc1300.pdf', 'search_text':'/usr/share/kicad/symbols/Amplifier_Difference.kicad_sym\nAMC1300DWV\n\nisolated difference amplifier', 'pins':[
            Pin(num='1',name='V+',func=pin_types.PWRIN,unit=1),
            Pin(num='2',name='+',func=pin_types.INPUT,unit=1),
            Pin(num='3',name='-',func=pin_types.INPUT,unit=1),
            Pin(num='4',name='V-',func=pin_types.PWRIN,unit=1),
            Pin(num='5',name='V-',func=pin_types.PWRIN,unit=1),
            Pin(num='6',name='-',func=pin_types.OUTPUT,unit=1),
            Pin(num='7',name='+',func=pin_types.OUTPUT,unit=1),
            Pin(num='8',name='V+',func=pin_types.PWRIN,unit=1)], 'unit_defs':[] }),
        Part(**{ 'name':'INA105KP', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'INA105KP'}), 'ref_prefix':'U', 'fplist':['', 'Package_DIP:DIP-8_W7.62mm'], 'footprint':'', 'keywords':'difference amplifier', 'description':'', 'datasheet':'https://www.ti.com/lit/ds/symlink/ina105.pdf', 'search_text':'/usr/share/kicad/symbols/Amplifier_Difference.kicad_sym\nINA105KP\n\ndifference amplifier', 'pins':[
            Pin(num='1',name='REF',func=pin_types.PASSIVE,unit=1),
            Pin(num='2',name='-',func=pin_types.PASSIVE,unit=1),
            Pin(num='3',name='+',func=pin_types.PASSIVE,unit=1),
            Pin(num='4',name='V-',func=pin_types.PWRIN,unit=1),
            Pin(num='5',name='SENSE',func=pin_types.PASSIVE,unit=1),
            Pin(num='6',name='~',func=pin_types.OUTPUT,unit=1),
            Pin(num='7',name='V+',func=pin_types.PWRIN,unit=1),
            Pin(num='8',name='~',func=pin_types.NOCONNECT,unit=1)], 'unit_defs':[] }),
        Part(**{ 'name':'INA105KU', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'INA105KU'}), 'ref_prefix':'U', 'fplist':['', 'Package_DIP:DIP-8_W7.62mm', 'Package_SO:SOIC-8_3.9x4.9mm_P1.27mm'], 'footprint':'', 'keywords':'difference amplifier', 'description':'', 'datasheet':'https://www.ti.com/lit/ds/symlink/ina105.pdf', 'search_text':'/usr/share/kicad/symbols/Amplifier_Difference.kicad_sym\nINA105KU\n\ndifference amplifier', 'pins':[
            Pin(num='1',name='REF',func=pin_types.PASSIVE,unit=1),
            Pin(num='2',name='-',func=pin_types.PASSIVE,unit=1),
            Pin(num='3',name='+',func=pin_types.PASSIVE,unit=1),
            Pin(num='4',name='V-',func=pin_types.PWRIN,unit=1),
            Pin(num='5',name='SENSE',func=pin_types.PASSIVE,unit=1),
            Pin(num='6',name='~',func=pin_types.OUTPUT,unit=1),
            Pin(num='7',name='V+',func=pin_types.PWRIN,unit=1),
            Pin(num='8',name='~',func=pin_types.NOCONNECT,unit=1)], 'unit_defs':[] }),
        Part(**{ 'name':'LM733H', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'LM733H'}), 'ref_prefix':'U', 'fplist':['Package_TO_SOT_THT:TO-5-10', 'Package_TO_SOT_THT:TO-5-10'], 'footprint':'Package_TO_SOT_THT:TO-5-10', 'keywords':'single differential video opamp', 'description':'', 'datasheet':'http://www.soemtron.org/downloads/disposals/lm733cn.pdf', 'search_text':'/usr/share/kicad/symbols/Amplifier_Difference.kicad_sym\nLM733H\n\nsingle differential video opamp', 'pins':[
            Pin(num='1',name='~',func=pin_types.INPUT,unit=1),
            Pin(num='10',name='2A',func=pin_types.PASSIVE,unit=1),
            Pin(num='2',name='~',func=pin_types.INPUT,unit=1),
            Pin(num='3',name='2B',func=pin_types.PASSIVE,unit=1),
            Pin(num='4',name='1B',func=pin_types.PASSIVE,unit=1),
            Pin(num='5',name='V-',func=pin_types.PWRIN,unit=1),
            Pin(num='6',name='~',func=pin_types.OUTPUT,unit=1),
            Pin(num='7',name='~',func=pin_types.OUTPUT,unit=1),
            Pin(num='8',name='V+',func=pin_types.PWRIN,unit=1),
            Pin(num='9',name='1A',func=pin_types.PASSIVE,unit=1)], 'unit_defs':[] }),
        Part(**{ 'name':'THS4521ID', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'THS4521ID'}), 'ref_prefix':'U', 'fplist':['Package_SO:VSSOP-8_3x3mm_P0.65mm', 'Package_SO:SOIC-8_3.9x4.9mm_P1.27mm'], 'footprint':'Package_SO:VSSOP-8_3x3mm_P0.65mm', 'keywords':'differential amplifier', 'description':'', 'datasheet':'https://www.ti.com/lit/ds/symlink/ths4521.pdf', 'search_text':'/usr/share/kicad/symbols/Amplifier_Difference.kicad_sym\nTHS4521ID\n\ndifferential amplifier', 'pins':[
            Pin(num='1',name='-',func=pin_types.INPUT,unit=1),
            Pin(num='2',name='V_{OCM}',func=pin_types.INPUT,unit=1),
            Pin(num='3',name='V_{S+}',func=pin_types.PWRIN,unit=1),
            Pin(num='4',func=pin_types.OUTPUT,unit=1),
            Pin(num='5',func=pin_types.OUTPUT,unit=1),
            Pin(num='6',name='V_{S-}',func=pin_types.PWRIN,unit=1),
            Pin(num='7',name='~{PD}',func=pin_types.INPUT,unit=1),
            Pin(num='8',name='+',func=pin_types.INPUT,unit=1)], 'unit_defs':[] })])


# -*- coding: utf-8 -*-

# The MIT License (MIT) - Copyright (c) Dave Vandenbout.

"""
Generate KiCad 8 netlist.
"""

import os.path
import time
import uuid
from simp_sexp import Sexp

from skidl.design_class import NetClass
from skidl.pckg_info import __version__
from skidl.scriptinfo import scriptinfo, get_script_dir
from skidl.utilities import export_to_all

# This UUID was generated using uuidgen for passing as the namespace argument to uuid.uuid5().
namespace_uuid = uuid.UUID("7026fcc6-e1a0-409e-aaf4-6a17ea82654f")


def gen_sheetpath(hierarchy):
    """
    Generate a sheetpath string from a hierarchical path tuple.

    A sheetpath is a string representation of the hierarchical path
    in a KiCad project. This function converts the given hierarchy
    tuple into a valid sheetpath format by joining the elements of the
    hierarchy tuple with '/' and ensuring it starts and ends with '/'.

    Args:
        hierarchy (tuple): A tuple of strings with the name of each level
            of the hierarchy.

    Returns:
        str: The generated sheetpath string. If the input hierarchy
             is empty or None, the function returns "/".
    """

    assert hierarchy[0] == "", "Top level of hierarchy must be an empty string."
    return f"{'/'.join(hierarchy)}/"


def gen_part_tstamp(part):
    """
    Generate a unique timestamp for a given part based on its hierarchical name.

    This function uses a UUID version 5 (SHA-1 hash) to create a deterministic
    and unique identifier for the part. The UUID is generated using a namespace
    UUID and the hierarchical name of the part.

    Args:
        part: An object representing the part. It is expected to have a
              'hiername' attribute that uniquely identifies the part
              within its hierarchy.

    Returns:
        str: A string representation of the generated UUID.
    """

    part_tstamp = str(uuid.uuid5(namespace_uuid, part.hiername))
    return part_tstamp


def gen_sheetpath_tstamp(hierarchy):
    """
    Generate a timestamp from a hierarchical path tuple.

    This function creates a unique timestamp for a hierarchical path
    in a KiCad project. If the hierarchy is empty, the timestamp
    will be "/". Otherwise, it generates a UUID for each
    entry of the tuple and combines them into a single timestamp.

    Args:
        hierarchy (tuple): A tuple of strings with the name of each level
            of the hierarchy.

    Returns:
        str: A timestamp for the sheetpath. For the root path, it returns "/".
             For other paths, it returns a UUID-based timestamp in the format
             "/<UUID>/<UUID>/.../<UUID>/", where each UUID corresponds to a
             segment of the sheetpath.
    """

    assert hierarchy[0] == "", "Top level of hierarchy must be an empty string."
    if len(hierarchy) == 1:
        tstamp = "/"
    else:
        tstamp = "/".join(
            [str(uuid.uuid5(namespace_uuid, level)) for level in hierarchy[1:]]
        )
        tstamp = "/" + tstamp + "/"
    return tstamp


def gen_netlist_sheet(hierarchy, number, src_file, **kwargs):
    """
    Generate a netlist sheet representation for a KiCad project.

    This function creates a hierarchical representation of a sheet in a KiCad
    netlist, including its path, timestamp, and title block information.

    Args:
        hierarchy (list): A list representing the hierarchical structure of the sheet.
        number (int): The sheet number in the hierarchy.
        src_file (str): The source file associated with the sheet.

    Returns:
        list: A nested list structure representing the netlist sheet, including
              its metadata and title block information.
    """
    sheetpath = gen_sheetpath(hierarchy)
    sheetpath_tstamp = gen_sheetpath_tstamp(hierarchy)

    if kwargs.get("track_abs_path", True):
        # If track_abs_path is True, use the absolute path of the source filename.
        src_file = os.path.abspath(src_file)
    else:
        # If track_abs_path is False, use the path of the source filename relative to the script.
        src_file = os.path.relpath(src_file, get_script_dir())

    sheet = Sexp([
        "sheet",
        ["number", number],
        ["name", sheetpath],
        ["tstamps", sheetpath_tstamp],
        [
            "title_block",
            ["title"],
            ["company"],
            ["rev"],
            ["date"],
            ["source", src_file],
            ["comment", ["number", "1"], ["value", ""]],
            ["comment", ["number", "2"], ["value", ""]],
            ["comment", ["number", "3"], ["value", ""]],
            ["comment", ["number", "4"], ["value", ""]],
            ["comment", ["number", "5"], ["value", ""]],
            ["comment", ["number", "6"], ["value", ""]],
            ["comment", ["number", "7"], ["value", ""]],
            ["comment", ["number", "8"], ["value", ""]],
            ["comment", ["number", "9"], ["value", ""]],
        ],
    ])

    return sheet


def gen_netlist_comp(part, **kwargs):
    """
    Generate a netlist component representation for a given part.

    This function takes a part object and generates a hierarchical representation
    of the part's attributes and metadata in a format suitable for inclusion in
    a KiCad netlist. The generated structure includes information such as the
    part's reference, value, footprint, description, datasheet, and additional
    fields.

    Args:
        part (Part): The part object containing attributes such as name, reference,
                     value, footprint, description, datasheet, and other metadata.

    Returns:
        list: A nested list structure representing the netlist component,
              including fields like reference, value, description, library source,
              sheetpath, timestamps, and custom fields.
    """

    part_name = part.name
    ref = part.ref
    value = part.value_to_str()
    footprint = getattr(part, "footprint", "")
    description = getattr(part, "description", "")
    datasheet = getattr(part, "datasheet", "")
    lib_filename = getattr(getattr(part, "lib", ""), "filename", "NO_LIB")

    # Embed the part hierarchy as a set of UUIDs into the sheetpath for each component.
    # This enables hierarchical selection in pcbnew.
    sheetpath = gen_sheetpath(part.hiertuple)
    sheetpath_tstamp = gen_sheetpath_tstamp(part.hiertuple)
    part_tstamp = gen_part_tstamp(part)

    part_classes = part.partclasses.by_priority()
    component_classes = Sexp(["component_classes"])
    for cls in reversed(part_classes):
        component_classes.append(Sexp(["class", cls.name]))

    fields = Sexp(["fields"])
    part_fields = list(part.fields.items())
    part_fields += list(
        {
            "Description": description,
            "Footprint": footprint,
            "Datasheet": datasheet,
        }.items()
    )
    part_fields.append(["SKiDL Tag", part.tag])
    if kwargs["track_src"]:
        part_fields.append(["SKiDL Line", part.src_line(kwargs["track_abs_path"])])
    for fld_name, fld_value in part_fields:
        if fld_value:
            field = Sexp(["field", ["name", fld_name], fld_value])
        else:
            field = Sexp(["field", ["name", fld_name]])
        fields.append(field)

    comp = Sexp([
        "comp",
        ["ref", ref],
        ["value", value],
        ["description", description],
        ["footprint", footprint],
        # ["datasheet", datasheet],
        fields,
        ["libsource", ["lib", lib_filename], ["part", part_name]],
        ["sheetpath", ["names", sheetpath], ["tstamps", sheetpath_tstamp]],
        component_classes,
        ["tstamps", part_tstamp],
    ])

    # If part has a 'dnp' attribute set to True, add the dnp property
    if getattr(part, "dnp", False):
        comp.append(Sexp(["property", ["name", "dnp"]]))
    
    # If part has an 'exclude_from_bom' attribute set to True, add the exclude_from_bom property
    if getattr(part, "exclude_from_bom", False):
        comp.append(Sexp(["property", ["name", "exclude_from_bom"]]))

    return comp


def gen_netlist_net(net, **kwargs):
    """
    Generate a netlist representation for a given net.

    This function creates a hierarchical list structure representing a net
    in a KiCad netlist. The netlist includes the net's code, name, and the
    associated pins sorted by their string representation.

    Args:
        net (Net): The net object containing information about the net's
                   code, name, and associated pins.

    Returns:
        list: A nested list structure representing the net, including
              its code, name, and associated pins with their part references
              and pin numbers.
    """
    net_classes = net.netclasses.by_priority()
    net_class_str = ",".join(reversed(net_classes))
    nt_lst = Sexp(["net", ["code", net.code], ["name", net.name], ["class", net_class_str]])
    for p in sorted(net.pins, key=str):
        _, _, pin_type = p.get_pin_info()
        nt_lst.append(["node", ["ref", p.part.ref], ["pin", p.num], ["pintype", pin_type]])

    return nt_lst


@export_to_all
def gen_netlist(circuit, **kwargs):
    """
    Generate a netlist for a given circuit.

    This function creates a netlist representation of the circuit, which includes
    information about components, nets, and design metadata. The netlist is formatted
    as a nested list structure and converted to S-expression format using the Sexp class.

    Args:
        circuit (Circuit): The circuit object containing parts, nets, and other
            design information.

    Returns:
        str: The netlist in S-expression format.

    Notes:
        - The function performs checks for empty footprints and randomly-assigned
          part tags to ensure the netlist is stable and usable for PCB design.
        - The netlist includes metadata such as the source file, date, and tool
          version.
        - Components and nets are sorted for consistent output.
        - The Sexp class is used to create a properly formatted S-expression.
    """

    # If track_src, track_abs_path is not specified in kwargs, use values from the circuit attributes.
    kwargs["track_src"] = kwargs.get("track_src", circuit.track_src)
    kwargs["track_abs_path"] = kwargs.get("track_abs_path", circuit.track_abs_path)

    # Check for some things that can cause problems if the netlist is
    # used to create a PCB.

    # Check for parts with no physical footprint to place on the PCB.
    circuit.check_for_empty_footprints()

    # Check for any missing tags since those will lead to
    # unstable associations between parts and PCB footprints.
    circuit.check_tags()

    # Add a Default netclass to the circuit if none exists.
    if "Default" not in circuit.netclasses:
        NetClass("Default", circuit=circuit, priority=0)

    # Add the Default netclass to all the nets.
    for net in circuit.get_nets():
        net.netclasses = "Default"

    scr_dict = scriptinfo()
    src_file = os.path.join(scr_dict["dir"], scr_dict["source"])
    if kwargs.get("track_abs_path", True):
        # If track_abs_path is True, use the absolute path of the source filename.
        src_file = os.path.abspath(src_file)
    else:
        # If track_abs_path is False, use the path of the source filename relative to the script.
        src_file = os.path.relpath(src_file, get_script_dir())

    date = time.strftime("%m/%d/%Y %I:%M %p")
    tool = f"SKiDL ({__version__})"

    sheets = Sexp()
    for num, node_name in enumerate(circuit.get_node_names(), 1):
        sheets.append(gen_netlist_sheet(node_name, num, src_file, **kwargs))

    components = Sexp()
    for p in sorted(circuit.parts, key=lambda p: str(p.ref)):
        components.append(gen_netlist_comp(p, **kwargs))

    nets = Sexp()
    sorted_nets = sorted(circuit.get_nets(), key=lambda n: str(n.name))
    for code, net in enumerate(sorted_nets, 1):
        net.code = code
        nets.append(gen_netlist_net(net, **kwargs))

    netlist = Sexp([
        "export",
        ["version", "D"],
        [
            "design",
            ["source", src_file],
            ["date", date],
            ["tool", tool],
            *sheets,
        ],
        ["components", *components],
        ["nets", *nets],
    ])

    # Add quotes to all strings following the initial keyword in each S-expression of the netlist.
    netlist.add_quotes(lambda s: True)

    # For some reason, KiCad's PCBNEW expects a space after the beginning export keyword
    # or else it rejects the netlist file.
    return netlist.to_str().replace("(export\n", "(export \n", 1)



from collections import defaultdict
from skidl import Pin, Part, Alias, SchLib, SKIDL, TEMPLATE

from skidl.pin import pin_types

SKIDL_lib_version = '0.0.1'

DSP_Motorola = SchLib(tool=SKIDL).add_parts(*[
        Part(**{ 'name':'DSP56301', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'DSP56301'}), 'ref_prefix':'U', 'fplist':['Package_QFP:PQFP-208_28x28mm_P0.5mm'], 'footprint':'Package_QFP:PQFP-208_28x28mm_P0.5mm', 'keywords':'DSP PCI', 'description':'', 'datasheet':'https://www.nxp.com/docs/en/data-sheet/DSP56301DS.pdf', 'search_text':'/usr/share/kicad/symbols/DSP_Motorola.kicad_sym\nDSP56301\n\nDSP PCI', 'pins':[
            Pin(num='1',name='AA0/RAS0',func=pin_types.OUTPUT,unit=1),
            Pin(num='10',name='/RESET',func=pin_types.INPUT,unit=1),
            Pin(num='100',name='D23',func=pin_types.BIDIR,unit=1),
            Pin(num='101',name='MODA/IRQA',func=pin_types.INPUT,unit=1),
            Pin(num='102',name='MODB/IRQB',func=pin_types.INPUT,unit=1),
            Pin(num='105',name='MODC/IRQC',func=pin_types.INPUT,unit=1),
            Pin(num='106',name='MODD/IRQD',func=pin_types.INPUT,unit=1),
            Pin(num='107',name='HAD31',func=pin_types.BIDIR,unit=1),
            Pin(num='108',name='HAD30',func=pin_types.BIDIR,unit=1),
            Pin(num='109',name='HAD29',func=pin_types.BIDIR,unit=1),
            Pin(num='11',name='VCCPLL',func=pin_types.PWRIN,unit=1),
            Pin(num='110',name='HAD28',func=pin_types.BIDIR,unit=1),
            Pin(num='111',name='VCC',func=pin_types.PWRIN,unit=1),
            Pin(num='112',name='GND',func=pin_types.PWRIN,unit=1),
            Pin(num='113',name='HAD27',func=pin_types.BIDIR,unit=1),
            Pin(num='114',name='HAD26',func=pin_types.BIDIR,unit=1),
            Pin(num='115',name='HAD25',func=pin_types.BIDIR,unit=1),
            Pin(num='116',name='HAD24',func=pin_types.BIDIR,unit=1),
            Pin(num='117',name='HC3/HBE3',func=pin_types.BIDIR,unit=1),
            Pin(num='118',name='HAD23',func=pin_types.BIDIR,unit=1),
            Pin(num='119',name='HAD22',func=pin_types.BIDIR,unit=1),
            Pin(num='12',name='PCAP',func=pin_types.INPUT,unit=1),
            Pin(num='120',name='HAD21',func=pin_types.BIDIR,unit=1),
            Pin(num='121',name='HAD20',func=pin_types.BIDIR,unit=1),
            Pin(num='122',name='VCC',func=pin_types.PWRIN,unit=1),
            Pin(num='123',name='GND',func=pin_types.PWRIN,unit=1),
            Pin(num='124',name='HAD19',func=pin_types.BIDIR,unit=1),
            Pin(num='125',name='HAD18',func=pin_types.BIDIR,unit=1),
            Pin(num='126',name='HAD17',func=pin_types.BIDIR,unit=1),
            Pin(num='127',name='HAD16',func=pin_types.BIDIR,unit=1),
            Pin(num='128',name='HC2/HBE2',func=pin_types.BIDIR,unit=1),
            Pin(num='129',name='HIDSEL',func=pin_types.INPUT,unit=1),
            Pin(num='13',name='GNDP',func=pin_types.PWRIN,unit=1),
            Pin(num='130',name='/HFRAME',func=pin_types.BIDIR,unit=1),
            Pin(num='131',name='VCC',func=pin_types.PWRIN,unit=1),
            Pin(num='132',name='GND',func=pin_types.PWRIN,unit=1),
            Pin(num='133',name='/HIRDY',func=pin_types.BIDIR,unit=1),
            Pin(num='134',name='/HTRDY',func=pin_types.BIDIR,unit=1),
            Pin(num='135',name='VCC',func=pin_types.PWRIN,unit=1),
            Pin(num='136',name='GND',func=pin_types.PWRIN,unit=1),
            Pin(num='137',name='PVCL',func=pin_types.INPUT,unit=1),
            Pin(num='138',name='/HDEVSEL',func=pin_types.BIDIR,unit=1),
            Pin(num='139',name='/HSTOP',func=pin_types.BIDIR,unit=1),
            Pin(num='14',name='GND1P',func=pin_types.PWRIN,unit=1),
            Pin(num='140',name='/HLOCK',func=pin_types.BIDIR,unit=1),
            Pin(num='141',name='/HPERR',func=pin_types.BIDIR,unit=1),
            Pin(num='142',name='/HSERR',func=pin_types.OUTPUT,unit=1),
            Pin(num='143',name='GND',func=pin_types.PWRIN,unit=1),
            Pin(num='144',name='VCC',func=pin_types.PWRIN,unit=1),
            Pin(num='145',name='HPAR',func=pin_types.BIDIR,unit=1),
            Pin(num='146',name='/HREQ',func=pin_types.OUTPUT,unit=1),
            Pin(num='147',name='/HRST',func=pin_types.INPUT,unit=1),
            Pin(num='148',name='HCLK',func=pin_types.INPUT,unit=1),
            Pin(num='149',name='/HGNT',func=pin_types.INPUT,unit=1),
            Pin(num='15',name='/BB',func=pin_types.BIDIR,unit=1),
            Pin(num='150',name='HC1/HBE1',func=pin_types.BIDIR,unit=1),
            Pin(num='151',name='HAD15',func=pin_types.BIDIR,unit=1),
            Pin(num='152',name='HAD14',func=pin_types.BIDIR,unit=1),
            Pin(num='153',name='HAD13',func=pin_types.BIDIR,unit=1),
            Pin(num='154',name='HAD12',func=pin_types.BIDIR,unit=1),
            Pin(num='155',name='GND',func=pin_types.PWRIN,unit=1),
            Pin(num='156',name='VCC',func=pin_types.PWRIN,unit=1),
            Pin(num='159',name='HAD11',func=pin_types.BIDIR,unit=1),
            Pin(num='16',name='/BG',func=pin_types.INPUT,unit=1),
            Pin(num='160',name='HAD10',func=pin_types.BIDIR,unit=1),
            Pin(num='161',name='HAD9',func=pin_types.BIDIR,unit=1),
            Pin(num='162',name='HAD8',func=pin_types.BIDIR,unit=1),
            Pin(num='163',name='HC0/HBE0',func=pin_types.BIDIR,unit=1),
            Pin(num='164',name='HAD7',func=pin_types.BIDIR,unit=1),
            Pin(num='165',name='HAD6',func=pin_types.BIDIR,unit=1),
            Pin(num='166',name='HAD5',func=pin_types.BIDIR,unit=1),
            Pin(num='167',name='HAD4',func=pin_types.BIDIR,unit=1),
            Pin(num='168',name='GND',func=pin_types.PWRIN,unit=1),
            Pin(num='169',name='VCC',func=pin_types.PWRIN,unit=1),
            Pin(num='17',name='/BR',func=pin_types.OUTPUT,unit=1),
            Pin(num='170',name='HAD3',func=pin_types.BIDIR,unit=1),
            Pin(num='171',name='HAD2',func=pin_types.BIDIR,unit=1),
            Pin(num='172',name='HAD1',func=pin_types.BIDIR,unit=1),
            Pin(num='173',name='HAD0',func=pin_types.BIDIR,unit=1),
            Pin(num='174',name='TIO2',func=pin_types.BIDIR,unit=1),
            Pin(num='175',name='TIO1',func=pin_types.BIDIR,unit=1),
            Pin(num='176',name='TIO0',func=pin_types.BIDIR,unit=1),
            Pin(num='177',name='PE0/RXD',func=pin_types.BIDIR,unit=1),
            Pin(num='178',name='PE2/SCLK',func=pin_types.BIDIR,unit=1),
            Pin(num='179',name='VCC',func=pin_types.PWRIN,unit=1),
            Pin(num='18',name='VCC',func=pin_types.PWRIN,unit=1),
            Pin(num='180',name='GND',func=pin_types.PWRIN,unit=1),
            Pin(num='181',name='/HINTA',func=pin_types.OUTPUT,unit=1),
            Pin(num='182',name='VCC',func=pin_types.PWRIN,unit=1),
            Pin(num='183',name='GND',func=pin_types.PWRIN,unit=1),
            Pin(num='184',name='PE1/TXD',func=pin_types.BIDIR,unit=1),
            Pin(num='185',name='PD2/SC12',func=pin_types.BIDIR,unit=1),
            Pin(num='186',name='PD1/SC11',func=pin_types.BIDIR,unit=1),
            Pin(num='187',name='PD0/SC10',func=pin_types.BIDIR,unit=1),
            Pin(num='188',name='PD5/STD1',func=pin_types.BIDIR,unit=1),
            Pin(num='189',name='PD3/SCK1',func=pin_types.BIDIR,unit=1),
            Pin(num='19',name='GND',func=pin_types.PWRIN,unit=1),
            Pin(num='190',name='PD4/SRD1',func=pin_types.BIDIR,unit=1),
            Pin(num='191',name='PC4/SRD0',func=pin_types.BIDIR,unit=1),
            Pin(num='192',name='PC3/SCK0',func=pin_types.BIDIR,unit=1),
            Pin(num='193',name='VCC',func=pin_types.PWRIN,unit=1),
            Pin(num='194',name='GND',func=pin_types.PWRIN,unit=1),
            Pin(num='195',name='PC5/STD0',func=pin_types.BIDIR,unit=1),
            Pin(num='196',name='PC0/SC00',func=pin_types.BIDIR,unit=1),
            Pin(num='197',name='PC1/SC01',func=pin_types.BIDIR,unit=1),
            Pin(num='198',name='PC2/SC02',func=pin_types.BIDIR,unit=1),
            Pin(num='199',name='/DE',func=pin_types.OUTPUT,unit=1),
            Pin(num='2',name='AA1/RAS1',func=pin_types.OUTPUT,unit=1),
            Pin(num='20',name='AA2/RAS2',func=pin_types.OUTPUT,unit=1),
            Pin(num='200',name='TMS',func=pin_types.INPUT,unit=1),
            Pin(num='201',name='TCK',func=pin_types.INPUT,unit=1),
            Pin(num='202',name='TDI',func=pin_types.INPUT,unit=1),
            Pin(num='203',name='TDO',func=pin_types.OUTPUT,unit=1),
            Pin(num='204',name='/TRST',func=pin_types.INPUT,unit=1),
            Pin(num='205',name='/BS',func=pin_types.OUTPUT,unit=1),
            Pin(num='206',name='/BL',func=pin_types.OUTPUT,unit=1),
            Pin(num='21',name='AA3/RAS3',func=pin_types.OUTPUT,unit=1),
            Pin(num='22',name='/WR',func=pin_types.OUTPUT,unit=1),
            Pin(num='23',name='/RD',func=pin_types.OUTPUT,unit=1),
            Pin(num='24',name='XTAL',func=pin_types.OUTPUT,unit=1),
            Pin(num='25',name='VCC',func=pin_types.PWRIN,unit=1),
            Pin(num='26',name='EXTAL',func=pin_types.INPUT,unit=1),
            Pin(num='27',name='GND',func=pin_types.PWRIN,unit=1),
            Pin(num='28',name='/BCLK',func=pin_types.OUTPUT,unit=1),
            Pin(num='29',name='A0',func=pin_types.OUTPUT,unit=1),
            Pin(num='3',name='VCC',func=pin_types.PWRIN,unit=1),
            Pin(num='30',name='A1',func=pin_types.OUTPUT,unit=1),
            Pin(num='31',name='GND',func=pin_types.PWRIN,unit=1),
            Pin(num='32',name='VCC',func=pin_types.PWRIN,unit=1),
            Pin(num='33',name='A2',func=pin_types.OUTPUT,unit=1),
            Pin(num='34',name='A3',func=pin_types.OUTPUT,unit=1),
            Pin(num='35',name='A4',func=pin_types.OUTPUT,unit=1),
            Pin(num='36',name='A5',func=pin_types.OUTPUT,unit=1),
            Pin(num='37',name='GND',func=pin_types.PWRIN,unit=1),
            Pin(num='38',name='VCC',func=pin_types.PWRIN,unit=1),
            Pin(num='39',name='A6',func=pin_types.OUTPUT,unit=1),
            Pin(num='4',name='GND',func=pin_types.PWRIN,unit=1),
            Pin(num='40',name='A7',func=pin_types.OUTPUT,unit=1),
            Pin(num='41',name='A8',func=pin_types.OUTPUT,unit=1),
            Pin(num='42',name='A9',func=pin_types.OUTPUT,unit=1),
            Pin(num='43',name='GND',func=pin_types.PWRIN,unit=1),
            Pin(num='44',name='VCC',func=pin_types.PWRIN,unit=1),
            Pin(num='45',name='A10',func=pin_types.OUTPUT,unit=1),
            Pin(num='46',name='A11',func=pin_types.OUTPUT,unit=1),
            Pin(num='47',name='A12',func=pin_types.OUTPUT,unit=1),
            Pin(num='48',name='A13',func=pin_types.OUTPUT,unit=1),
            Pin(num='49',name='GND',func=pin_types.PWRIN,unit=1),
            Pin(num='5',name='CLKOUT',func=pin_types.OUTPUT,unit=1),
            Pin(num='50',name='VCC',func=pin_types.PWRIN,unit=1),
            Pin(num='51',name='A14',func=pin_types.OUTPUT,unit=1),
            Pin(num='52',name='A15',func=pin_types.OUTPUT,unit=1),
            Pin(num='55',name='A16',func=pin_types.OUTPUT,unit=1),
            Pin(num='56',name='A17',func=pin_types.OUTPUT,unit=1),
            Pin(num='57',name='GND',func=pin_types.PWRIN,unit=1),
            Pin(num='58',name='VCC',func=pin_types.PWRIN,unit=1),
            Pin(num='59',name='A18',func=pin_types.OUTPUT,unit=1),
            Pin(num='6',name='BCLK',func=pin_types.OUTPUT,unit=1),
            Pin(num='60',name='A19',func=pin_types.OUTPUT,unit=1),
            Pin(num='61',name='A20',func=pin_types.OUTPUT,unit=1),
            Pin(num='62',name='A21',func=pin_types.OUTPUT,unit=1),
            Pin(num='63',name='GND',func=pin_types.PWRIN,unit=1),
            Pin(num='64',name='VCC',func=pin_types.PWRIN,unit=1),
            Pin(num='65',name='A22',func=pin_types.OUTPUT,unit=1),
            Pin(num='66',name='A23',func=pin_types.OUTPUT,unit=1),
            Pin(num='67',name='D0',func=pin_types.BIDIR,unit=1),
            Pin(num='68',name='D1',func=pin_types.BIDIR,unit=1),
            Pin(num='69',name='D2',func=pin_types.BIDIR,unit=1),
            Pin(num='7',name='/CAS',func=pin_types.OUTPUT,unit=1),
            Pin(num='70',name='GND',func=pin_types.PWRIN,unit=1),
            Pin(num='71',name='VCC',func=pin_types.PWRIN,unit=1),
            Pin(num='72',name='D3',func=pin_types.BIDIR,unit=1),
            Pin(num='73',name='D4',func=pin_types.BIDIR,unit=1),
            Pin(num='74',name='D5',func=pin_types.BIDIR,unit=1),
            Pin(num='75',name='D6',func=pin_types.BIDIR,unit=1),
            Pin(num='76',name='D7',func=pin_types.BIDIR,unit=1),
            Pin(num='77',name='D8',func=pin_types.BIDIR,unit=1),
            Pin(num='78',name='GND',func=pin_types.PWRIN,unit=1),
            Pin(num='79',name='VCC',func=pin_types.PWRIN,unit=1),
            Pin(num='8',name='/TA',func=pin_types.INPUT,unit=1),
            Pin(num='80',name='GND',func=pin_types.PWRIN,unit=1),
            Pin(num='81',name='VCC',func=pin_types.PWRIN,unit=1),
            Pin(num='82',name='D9',func=pin_types.BIDIR,unit=1),
            Pin(num='83',name='D10',func=pin_types.BIDIR,unit=1),
            Pin(num='84',name='D11',func=pin_types.BIDIR,unit=1),
            Pin(num='85',name='D12',func=pin_types.BIDIR,unit=1),
            Pin(num='86',name='D13',func=pin_types.BIDIR,unit=1),
            Pin(num='87',name='D14',func=pin_types.BIDIR,unit=1),
            Pin(num='88',name='GND',func=pin_types.PWRIN,unit=1),
            Pin(num='89',name='VCC',func=pin_types.PWRIN,unit=1),
            Pin(num='9',name='PINIT/NMI',func=pin_types.INPUT,unit=1),
            Pin(num='90',name='D15',func=pin_types.BIDIR,unit=1),
            Pin(num='91',name='D16',func=pin_types.BIDIR,unit=1),
            Pin(num='92',name='D17',func=pin_types.BIDIR,unit=1),
            Pin(num='93',name='D18',func=pin_types.BIDIR,unit=1),
            Pin(num='94',name='D19',func=pin_types.BIDIR,unit=1),
            Pin(num='95',name='D20',func=pin_types.BIDIR,unit=1),
            Pin(num='96',name='GND',func=pin_types.PWRIN,unit=1),
            Pin(num='97',name='VCC',func=pin_types.PWRIN,unit=1),
            Pin(num='98',name='D21',func=pin_types.BIDIR,unit=1),
            Pin(num='99',name='D22',func=pin_types.BIDIR,unit=1)], 'unit_defs':[] })])